import re
from os import environ as env
from pathlib import Path


def find_pgpass():
    """Returns the path to the PGPASS file.
    PG Reference: https://www.postgresql.org/docs/current/libpq-pgpass.html
    """
    if "PGPASSFILE" in env:
        return Path(env["PGPASSFILE"])

    homepgpass = Path.home() / ".pgpass"
    if homepgpass.is_file():
        return homepgpass
    # if no pgpass is found, create one with file permissions as recommended
    # by the reference above.
    homepgpass.touch(mode=0o600)
    return homepgpass


def read_pgpass():
    path = find_pgpass()
    return open(path).read(), path


def write_pgpass(path, content):
    with open(path, "w") as f:
        f.write(content)


def update_pgpass_str(pgpass, hosts, access_token):
    for host in hosts:
        newline = f"{host}:*:*:*:{access_token}"
        p = re.compile(fr"{host}:\*:\*:\*:.*")
        pgpass, num_replaces = p.subn(newline, pgpass)
        if num_replaces == 0:
            # match is not found, add line on top
            pgpass = "\n".join((newline, pgpass))
    return pgpass


def update_pgpass(access_token, hosts):
    pgpass, path = read_pgpass()
    newpgpass = update_pgpass_str(pgpass, hosts, access_token)
    write_pgpass(path, newpgpass)
